@@ -3,7 +3,7 @@
 #
 # Chris Lumens <clumens@redhat.com>
 #
-# Copyright 2005, 2006, 2007, 2008, 2011 Red Hat, Inc.
+# Copyright 2005-2016 Red Hat, Inc.
 #
 # This copyrighted material is made available to anyone wishing to use, modify,
 # copy, or redistribute it subject to the terms and conditions of the GNU
@@ -17,54 +17,50 @@
 # Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
 # trademarks that are incorporated in the source code or documentation are not
 # subject to the GNU General Public License and may only be used or replicated
-# with the express permission of Red Hat, Inc. 
+# with the express permission of Red Hat, Inc.
 #
 """
 Main kickstart file processing module.
 
 This module exports several important classes:
 
-    Script - Representation of a single %pre, %post, or %traceback script.
+    Script - Representation of a single %pre, %pre-install, %post, or %traceback script.
 
     Packages - Representation of the %packages section.
 
     KickstartParser - The kickstart file parser state machine.
 """
-from collections import Iterator
+from collections.abc import Iterator
+
 import os
 import shlex
 import sys
-import tempfile
-from copy import copy
-from optparse import *
-from urlgrabber import urlread
-import urlgrabber.grabber as grabber
+import warnings
 
-import constants
-from errors import KickstartError, KickstartParseError, KickstartValueError, formatErrorMsg
-from ko import KickstartObject
-from sections import *
-import version
+from pykickstart import constants, version
+from pykickstart.errors import KickstartError, KickstartParseError, KickstartParseWarning
+from pykickstart.ko import KickstartObject
+from pykickstart.options import KSOptionParser
+from pykickstart.load import load_to_str
+from pykickstart.sections import PackageSection, PreScriptSection, PreInstallScriptSection, \
+                                 PostScriptSection, TracebackScriptSection, OnErrorScriptSection, \
+                                 NullSection, RunScriptSection, PostUmountScriptSection, TpkPackageSection
 
-import gettext
-_ = lambda x: gettext.ldgettext("pykickstart", x)
+from pykickstart.i18n import _
 
 STATE_END = "end"
 STATE_COMMANDS = "commands"
 
-ver = version.DEVEL
-
-def _preprocessStateMachine (lineIter):
+def _preprocessStateMachine(lineIter):
     l = None
     lineno = 0
+    retval = ""
 
-    # Now open an output kickstart file that we are going to write to one
-    # line at a time.
-    (outF, outName) = tempfile.mkstemp("-ks.cfg", "", "/tmp")
+    retval = retval.encode(sys.getdefaultencoding())
 
     while True:
         try:
-            l = lineIter.next()
+            l = next(lineIter)
         except StopIteration:
             break
 
@@ -73,68 +69,86 @@ def _preprocessStateMachine (lineIter):
             break
 
         lineno += 1
-        url = None
+        ksurl = None
 
         ll = l.strip()
         if not ll.startswith("%ksappend"):
-            os.write(outF, l)
+            retval += l.encode(sys.getdefaultencoding())
             continue
 
         # Try to pull down the remote file.
         try:
             ksurl = ll.split(' ')[1]
         except:
-            raise KickstartParseError (formatErrorMsg(lineno, msg=_("Illegal url for %%ksappend: %s") % ll))
+            raise KickstartParseError(_("Illegal url for %%ksappend: %s") % ll, lineno=lineno)
 
         try:
-            url = grabber.urlopen(ksurl)
-        except grabber.URLGrabError as e:
-            raise KickstartError (formatErrorMsg(lineno, msg=_("Unable to open %%ksappend file: %s") % e.strerror))
-        else:
-            # Sanity check result.  Sometimes FTP doesn't catch a file
-            # is missing.
-            try:
-                if url.size < 1:
-                    raise KickstartError (formatErrorMsg(lineno, msg=_("Unable to open %%ksappend file")))
-            except:
-                raise KickstartError (formatErrorMsg(lineno, msg=_("Unable to open %%ksappend file")))
-
+            contents = load_to_str(ksurl)
+        except KickstartError as e:
+            raise KickstartError(_("Unable to open %%ksappend file: %s") % str(e), lineno=lineno)
         # If that worked, write the remote file to the output kickstart
-        # file in one burst.  Then close everything up to get ready to
-        # read ahead in the input file.  This allows multiple %ksappend
-        # lines to exist.
-        if url is not None:
-            os.write(outF, url.read())
-            url.close()
+        # file in one burst.  This allows multiple %ksappend lines to
+        # exist.
+        if contents is not None:
+            retval += contents.encode(sys.getdefaultencoding())
 
-    # All done - close the temp file and return its location.
-    os.close(outF)
-    return outName
+    return retval
 
-def preprocessFromString (s):
-    """Preprocess the kickstart file, provided as the string str.  This
-        method is currently only useful for handling %ksappend lines,
-        which need to be fetched before the real kickstart parser can be
-        run.  Returns the location of the complete kickstart file.
+def preprocessFromStringToString(s):
+    """Preprocess the kickstart file, provided as the string s.  This
+       method is currently only useful for handling %ksappend lines, which
+       need to be fetched before the real kickstart parser can be run.
+       Returns the complete kickstart file as a string.
     """
     i = iter(s.splitlines(True) + [""])
-    rc = _preprocessStateMachine (i.next)
-    return rc
+    return _preprocessStateMachine(i)
 
-def preprocessKickstart (f):
-    """Preprocess the kickstart file, given by the filename file.  This
-        method is currently only useful for handling %ksappend lines,
-        which need to be fetched before the real kickstart parser can be
-        run.  Returns the location of the complete kickstart file.
+def preprocessKickstartToString(f):
+    """Preprocess the kickstart file, given by the filename f.  This
+       method is currently only useful for handling %ksappend lines,
+       which need to be fetched before the real kickstart parser can be
+       run.  Returns the complete kickstart file as a string.
     """
     try:
-        fh = urlopen(f)
-    except grabber.URLGrabError as e:
-        raise KickstartError (formatErrorMsg(0, msg=_("Unable to open input kickstart file: %s") % e.strerror))
+        contents = load_to_str(f)
+    except KickstartError as e:
+        raise KickstartError(_("Unable to open input kickstart file: %s") % str(e), lineno=0)
 
-    rc = _preprocessStateMachine (iter(fh.readlines()))
-    fh.close()
-    return rc
+    return _preprocessStateMachine(iter(contents.splitlines(True)))
+
+def preprocessFromString(s):
+    """Preprocess the kickstart file, provided as the string s.  This
+       method is currently only useful for handling %ksappend lines,
+       which need to be fetched before the real kickstart parser can be
+       run.  Returns the location of the complete kickstart file.
+    """
+    s = preprocessFromStringToString(s)
+    if s.strip():
+        import tempfile
+        (outF, outName) = tempfile.mkstemp(suffix="-ks.cfg")
+
+        os.write(outF, s)
+        os.close(outF)
+        return outName
+
+    return None
+
+def preprocessKickstart(f):
+    """Preprocess the kickstart file, given by the filename f.  This
+       method is currently only useful for handling %ksappend lines,
+       which need to be fetched before the real kickstart parser can be
+       run.  Returns the location of the complete kickstart file.
+    """
+    s = preprocessKickstartToString(f)
+    if s.strip():
+        import tempfile
+        (outF, outName) = tempfile.mkstemp(suffix="-ks.cfg")
+
+        os.write(outF, s)
+        os.close(outF)
+        return outName
+
+    return None
 
 class PutBackIterator(Iterator):
     def __init__(self, iterable):
@@ -153,33 +167,44 @@ class PutBackIterator(Iterator):
             self._buf = None
             return retval
         else:
-            return self._iterable.next()
+            return next(self._iterable)
+
+    def __next__(self):
+        return self.next()                          # pylint: disable=not-callable
 
 ###
 ### SCRIPT HANDLING
 ###
 class Script(KickstartObject):
+    _ver = version.DEVEL
+
     """A class representing a single kickstart script.  If functionality beyond
        just a data representation is needed (for example, a run method in
        anaconda), Script may be subclassed.  Although a run method is not
        provided, most of the attributes of Script have to do with running the
        script.  Instances of Script are held in a list by the Version object.
     """
-    def __init__(self, script, *args , **kwargs):
+    def __init__(self, script, *args, **kwargs):
         """Create a new Script instance.  Instance attributes:
 
-           errorOnFail -- If execution of the script fails, should anaconda
-                          stop, display an error, and then reboot without
-                          running any other scripts?
-           inChroot    -- Does the script execute in anaconda's chroot
-                          environment or not?
-           interp      -- The program that should be used to interpret this
-                          script.
-           lineno      -- The line number this script starts on.
-           logfile     -- Where all messages from the script should be logged.
-           script      -- A string containing all the lines of the script.
-           type        -- The type of the script, which can be KS_SCRIPT_* from
-                          pykickstart.constants.
+           :keyword errorOnFail: If execution of the script fails, should anaconda
+                                 stop, display an error, and then reboot without
+                                 running any other scripts?
+
+           :keyword inChroot: Does the script execute in anaconda's chroot
+                              environment or not?
+
+           :keyword interp: The program that should be used to interpret this
+                            script.
+
+           :keyword lineno: The line number this script starts on.
+
+           :keyword logfile: Where all messages from the script should be logged.
+
+           :keyword script: A string containing all the lines of the script.
+
+           :keyword type: The type of the script, which can be KS_SCRIPT_* from
+                          :mod:`pykickstart.constants`.
         """
         KickstartObject.__init__(self, *args, **kwargs)
         self.script = "".join(script)
@@ -201,36 +226,39 @@ class Script(KickstartObject):
             retval += '\n%post'
         elif self.type == constants.KS_SCRIPT_TRACEBACK:
             retval += '\n%traceback'
+        elif self.type == constants.KS_SCRIPT_PREINSTALL:
+            retval += '\n%pre-install'
+        elif self.type == constants.KS_SCRIPT_ONERROR:
+            retval += '\n%onerror'
         elif self.type == constants.KS_SCRIPT_RUN:
             retval += '\n%runscript'
         elif self.type == constants.KS_SCRIPT_UMOUNT:
             retval += '\n%post-umount'
 
-        if self.interp != "/bin/sh" and self.interp != "":
+        if self.interp != "/bin/sh" and self.interp:
             retval += " --interpreter=%s" % self.interp
         if self.type == constants.KS_SCRIPT_POST and not self.inChroot:
             retval += " --nochroot"
-        if self.logfile != None:
-            retval += " --logfile %s" % self.logfile
+        if self.logfile is not None:
+            retval += " --logfile=%s" % self.logfile
         if self.errorOnFail:
             retval += " --erroronfail"
 
         if self.script.endswith("\n"):
-            if ver >= version.F8:
+            if self._ver >= version.F8:
                 return retval + "\n%s%%end\n" % self.script
             else:
-                return retval + "\n%s\n" % self.script
+                return retval + "\n%s" % self.script
         else:
-            if ver >= version.F8:
+            if self._ver >= version.F8:
                 return retval + "\n%s\n%%end\n" % self.script
             else:
                 return retval + "\n%s\n" % self.script
 
-
 ##
 ## PACKAGE HANDLING
 ##
-class Group:
+class Group(KickstartObject):
     """A class representing a single group in the %packages section."""
     def __init__(self, name="", include=constants.GROUP_DEFAULT):
         """Create a new Group instance.  Instance attributes:
@@ -239,6 +267,7 @@ class Group:
            include -- The level of how much of the group should be included.
                       Values can be GROUP_* from pykickstart.constants.
         """
+        KickstartObject.__init__(self)
         self.name = name
         self.include = include
 
@@ -251,21 +280,41 @@ class Group:
         else:
             return "@%s" % self.name
 
-    def __cmp__(self, other):
-        if self.name < other.name:
-            return -1
-        elif self.name > other.name:
-            return 1
-        return 0
+    def __lt__(self, other):
+        return self.name < other.name
+
+    def __le__(self, other):
+        return self.name <= other.name
+
+    def __eq__(self, other):
+        return self.name == other.name
+
+    def __ne__(self, other):
+        return self.name != other.name
+
+    def __gt__(self, other):
+        return self.name > other.name
+
+    def __ge__(self, other):
+        return self.name >= other.name
+
+    __hash__ = KickstartObject.__hash__
 
 class Packages(KickstartObject):
+    _ver = version.DEVEL
+
     """A class representing the %packages section of the kickstart file."""
     def __init__(self, *args, **kwargs):
         """Create a new Packages instance.  Instance attributes:
 
            addBase       -- Should the Base group be installed even if it is
                             not specified?
+           nocore        -- Should the Core group be skipped?  This results in
+                            a %packages section that basically only installs the
+                            packages you list, and may not be a usable system.
            default       -- Should the default package set be selected?
+           environment   -- What base environment should be selected?  Only one
+                            may be chosen at a time.
            excludedList  -- A list of all the packages marked for exclusion in
                             the %packages section, without the leading minus
                             symbol.
@@ -280,52 +329,47 @@ class Packages(KickstartObject):
            handleMissing -- If unknown packages are specified in the %packages
                             section, should it be ignored or not?  Values can
                             be KS_MISSING_* from pykickstart.constants.
+           handleBroken  -- If packages with conflicts are specified in the
+                            %packages section, should it be ignored or not?
+                            Values can be KS_BROKEN_* from pykickstart.constants.
            packageList   -- A list of all the packages specified in the
                             %packages section.
            instLangs     -- A list of languages to install.
+           multiLib      -- Whether to use yum's "all" multilib policy.
+           excludeWeakdeps -- Whether to exclude weak dependencies.
+           timeout       -- Number of seconds to wait for a connection before
+                            yum's or dnf's timing out or None.
+           retries       -- Number of times yum's or dnf's attempt to retrieve
+                            a file should retry before returning an error.
+           seen          -- If %packages was ever used in the kickstart file,
+                            this attribute will be set to True.
+
         """
         KickstartObject.__init__(self, *args, **kwargs)
 
         self.addBase = True
+        self.nocore = False
         self.default = False
+        self.environment = None
         self.excludedList = []
         self.excludedGroupList = []
         self.excludeDocs = False
         self.groupList = []
         self.handleMissing = constants.KS_MISSING_PROMPT
+        self.handleBroken = constants.KS_BROKEN_REPORT
         self.packageList = []
         self.tpk_packageList = []
         self.instLangs = None
+        self.multiLib = False
+        self.excludeWeakdeps = False
+        self.timeout = None
+        self.retries = None
+        self.seen = False
 
     def __str__(self):
         """Return a string formatted for output to a kickstart file."""
-        pkgs = ""
-
-        if not self.default:
-            grps = self.groupList
-            grps.sort()
-            for grp in grps:
-                pkgs += "%s\n" % grp.__str__()
-
-            p = self.packageList
-            p.sort()
-            for pkg in p:
-                pkgs += "%s\n" % pkg
-
-            grps = self.excludedGroupList
-            grps.sort()
-            for grp in grps:
-                pkgs += "-%s\n" % grp.__str__()
-
-            p = self.excludedList
-            p.sort()
-            for pkg in p:
-                pkgs += "-%s\n" % pkg
-
-            if pkgs == "":
-                return ""
-
-        retval = "\n%packages"
+        pkgs = self._processPackagesContent()
+        retval = ""
 
         if self.default:
             retval += " --default"
@@ -333,38 +377,87 @@ class Packages(KickstartObject):
             retval += " --excludedocs"
         if not self.addBase:
             retval += " --nobase"
+        if self.nocore:
+            retval += " --nocore"
         if self.handleMissing == constants.KS_MISSING_IGNORE:
             retval += " --ignoremissing"
-        if self.instLangs:
-            retval += " --instLangs=%s" % self.instLangs
+        if self.handleBroken == constants.KS_BROKEN_IGNORE:
+            retval += " --ignorebroken"
+        if self.instLangs is not None:
+            retval += " --inst-langs=%s" % self.instLangs
+        if self.multiLib:
+            retval += " --multilib"
+        if self.excludeWeakdeps:
+            retval += " --exclude-weakdeps"
+        if self.timeout is not None:
+            retval += " --timeout=%d" % self.timeout
+        if self.retries is not None:
+            retval += " --retries=%d" % self.retries
 
-        if ver >= version.F8:
-            return retval + "\n" + pkgs + "\n%end\n"
+        if retval == "" and pkgs == "" and not self.seen:
+            return ""
+
+        if self._ver >= version.F8:
+            return "\n%packages" + retval + "\n" + pkgs + "\n%end\n"
         else:
-            return retval + "\n" + pkgs + "\n"
+            return "\n%packages" + retval + "\n" + pkgs + "\n"
 
-    def _processGroup (self, line):
-        op = OptionParser()
-        op.add_option("--nodefaults", action="store_true", default=False)
-        op.add_option("--optional", action="store_true", default=False)
+    def _processPackagesContent(self):
+        pkgs = ""
 
-        (opts, extra) = op.parse_args(args=line.split())
+        if not self.default:
+            if self.environment:
+                pkgs += "@^%s\n" % self.environment
 
-        if opts.nodefaults and opts.optional:
-            raise KickstartValueError (_("Group cannot specify both --nodefaults and --optional"))
+        grps = self.groupList
+        grps.sort()
+        for grp in grps:
+            pkgs += "%s\n" % grp.__str__()
+
+        p = self.packageList
+        p.sort()
+        for pkg in p:
+            pkgs += "%s\n" % pkg
+
+        grps = self.excludedGroupList
+        grps.sort()
+        for grp in grps:
+            pkgs += "-%s\n" % grp.__str__()
+
+        p = self.excludedList
+        p.sort()
+        for pkg in p:
+            pkgs += "-%s\n" % pkg
+
+        return pkgs
+
+    def _processGroup(self, line):
+        op = KSOptionParser(prog="", description="", version=version.DEVEL)
+        op.add_argument("--nodefaults", action="store_true", default=False,
+                        help="", version=version.DEVEL)
+        op.add_argument("--optional", action="store_true", default=False,
+                        help="", version=version.DEVEL)
+
+        (ns, extra) = op.parse_known_args(args=line.split())
+
+        if ns.nodefaults and ns.optional:
+            raise KickstartParseError(_("Group cannot specify both --nodefaults and --optional"))
 
         # If the group name has spaces in it, we have to put it back together
         # now.
         grp = " ".join(extra)
 
-        if opts.nodefaults:
+        if grp in [g.name for g in self.groupList]:
+            return
+
+        if ns.nodefaults:
             self.groupList.append(Group(name=grp, include=constants.GROUP_REQUIRED))
-        elif opts.optional:
+        elif ns.optional:
             self.groupList.append(Group(name=grp, include=constants.GROUP_ALL))
         else:
             self.groupList.append(Group(name=grp, include=constants.GROUP_DEFAULT))
 
-    def add (self, pkgList):
+    def add(self, pkgList):
         """Given a list of lines from the input file, strip off any leading
            symbols and add the result to the appropriate list.
         """
@@ -378,10 +471,14 @@ class Packages(KickstartObject):
         for pkg in pkgList:
             stripped = pkg.strip()
 
-            if stripped[0] == "@":
+            if stripped[0:2] == "@^":
+                self.environment = stripped[2:]
+            elif stripped[0] == "@":
                 self._processGroup(stripped[1:])
             elif stripped[0] == "-":
-                if stripped[1] == "@":
+                if stripped[1:3] == "@^" and self.environment == stripped[3:]:
+                    self.environment = None
+                elif stripped[1] == "@":
                     excludedGroupList.append(Group(name=stripped[2:]))
                 else:
                     newExcludedSet.add(stripped[1:])
@@ -390,12 +487,12 @@ class Packages(KickstartObject):
 
         # Groups have to be excluded in two different ways (note: can't use
         # sets here because we have to store objects):
-        excludedGroupNames = map(lambda g: g.name, excludedGroupList)
+        excludedGroupNames = [g.name for g in excludedGroupList]
 
         # First, an excluded group may be cancelling out a previously given
         # one.  This is often the case when using %include.  So there we should
         # just remove the group from the list.
-        self.groupList = filter(lambda g: g.name not in excludedGroupNames, self.groupList)
+        self.groupList = [g for g in self.groupList if g.name not in excludedGroupNames]
 
         # Second, the package list could have included globs which are not
         # processed by pykickstart.  In that case we need to preserve a list of
@@ -406,8 +503,9 @@ class Packages(KickstartObject):
         existingPackageSet = (existingPackageSet - newExcludedSet) | newPackageSet
         existingExcludedSet = (existingExcludedSet - existingPackageSet) | newExcludedSet
 
-        self.packageList = list(existingPackageSet)
-        self.excludedList = list(existingExcludedSet)
+        # FIXME: figure these types out
+        self.packageList = sorted(existingPackageSet)
+        self.excludedList = sorted(existingExcludedSet)
 
 class TpkPackages(KickstartObject):
     """A class representing the %tpk_packages section of the kickstart file."""
@@ -431,15 +529,15 @@ class TpkPackages(KickstartObject):
 ###
 ### PARSER
 ###
-class KickstartParser:
+class KickstartParser(object):
     """The kickstart file parser class as represented by a basic state
        machine.  To create a specialized parser, make a subclass and override
        any of the methods you care about.  Methods that don't need to do
        anything may just pass.  However, _stateMachine should never be
        overridden.
     """
-    def __init__ (self, handler, followIncludes=True, errorsAreFatal=True,
-                  missingIncludeIsFatal=True):
+    def __init__(self, handler, followIncludes=True, errorsAreFatal=True,
+                 missingIncludeIsFatal=True, unknownSectionIsFatal=True):
         """Create a new KickstartParser instance.  Instance attributes:
 
            errorsAreFatal        -- Should errors cause processing to halt, or
@@ -455,21 +553,26 @@ class KickstartParser:
                                     will be executed.
            missingIncludeIsFatal -- Should missing include files be fatal, even
                                     if errorsAreFatal is False?
+           unknownSectionIsFatal -- Should an unknown %section be fatal?  Not all
+                                    sections are handled by pykickstart.  Some are
+                                    user-defined, so there should be a way to have
+                                    pykickstart ignore them.
         """
         self.errorsAreFatal = errorsAreFatal
+        self.errorsCount = 0
         self.followIncludes = followIncludes
         self.handler = handler
         self.currentdir = {}
         self.missingIncludeIsFatal = missingIncludeIsFatal
+        self.unknownSectionIsFatal = unknownSectionIsFatal
 
         self._state = STATE_COMMANDS
         self._includeDepth = 0
         self._line = ""
 
         self.version = self.handler.version
-
-        global ver
-        ver = self.version
+        Script._ver = self.version
+        Packages._ver = self.version
 
         self._sections = {}
         self.setupSections()
@@ -485,17 +588,15 @@ class KickstartParser:
         """
         return self._sections[s]
 
-    def handleCommand (self, lineno, args):
+    def handleCommand(self, lineno, args):
         """Given the list of command and arguments, call the Version's
            dispatcher method to handle the command.  Returns the command or
            data object returned by the dispatcher.  This method may be
            overridden in a subclass if necessary.
         """
         if self.handler:
-            self.handler.currentCmd = args[0]
             self.handler.currentLine = self._line
             retval = self.handler.dispatcher(args, lineno)
-
             return retval
 
     def registerSection(self, obj):
@@ -505,10 +606,10 @@ class KickstartParser:
            handle it.
         """
         if not obj.sectionOpen:
-            raise TypeError ("no sectionOpen given for section %s" % obj)
+            raise TypeError("no sectionOpen given for section %s" % obj)
 
         if not obj.sectionOpen.startswith("%"):
-            raise TypeError ("section %s tag does not start with a %%" % obj.sectionOpen)
+            raise TypeError("section %s tag does not start with a %%" % obj.sectionOpen)
 
         self._sections[obj.sectionOpen] = obj
 
@@ -527,18 +628,18 @@ class KickstartParser:
 
         # Save the platform for s-c-kickstart.
         if line[:10] == "#platform=":
-            self.handler.platform = self._line[11:]
+            self.handler.platform = self._line[10:].strip()
 
     def _readSection(self, lineIter, lineno):
         obj = self._sections[self._state]
 
         while True:
             try:
-                line = lineIter.next()
-                if line == "":
+                line = next(lineIter)
+                if line == "" and self._includeDepth == 0:
                     # This section ends at the end of the file.
                     if self.version >= version.F8:
-                        raise KickstartParseError (formatErrorMsg(lineno, msg=_("Section does not end with %%end.")))
+                        raise KickstartParseError(_("Section %s does not end with %%end.") % obj.sectionOpen, lineno=lineno)
 
                     self._finalize(obj)
             except StopIteration:
@@ -551,19 +652,35 @@ class KickstartParser:
             if self._isBlankOrComment(line) and not obj.allLines:
                 continue
 
-            if line.startswith("%"):
+            if line.lstrip().startswith("%"):
+                # If we're in a script, the line may begin with "%something"
+                # that's not the start of any section we recognize, but still
+                # valid for that script.  So, don't do the split below unless
+                # we're sure.
+                possibleSectionStart = line.split()[0]
+                if not self._validState(possibleSectionStart) \
+                   and possibleSectionStart not in ("%end", "%include"):
+                    obj.handleLine(line)
+                    continue
+
                 args = shlex.split(line)
 
                 if args and args[0] == "%end":
                     # This is a properly terminated section.
                     self._finalize(obj)
                     break
+                elif args and args[0] == "%include":
+                    if len(args) == 1 or not args[1]:
+                        raise KickstartParseError(lineno=lineno)
+
+                    self._handleInclude(args[1])
+                    continue
                 elif args and args[0] == "%ksappend":
                     continue
-                elif args and (self._validState(args[0]) or args[0] in ["%include", "%ksappend"]):
+                elif args and self._validState(args[0]):
                     # This is an unterminated section.
                     if self.version >= version.F8:
-                        raise KickstartParseError (formatErrorMsg(lineno, msg=_("Section does not end with %%end.")))
+                        raise KickstartParseError(_("Section %s does not end with %%end.") % obj.sectionOpen, lineno=lineno)
 
                     # Finish up.  We do not process the header here because
                     # kicking back out to STATE_COMMANDS will ensure that happens.
@@ -580,7 +697,7 @@ class KickstartParser:
 
     def _validState(self, st):
         """Is the given section tag one that has been registered with the parser?"""
-        return st in self._sections.keys()
+        return st in list(self._sections.keys())
 
     def _tryFunc(self, fn):
         """Call the provided function (which doesn't take any arguments) and
@@ -589,15 +706,34 @@ class KickstartParser:
         """
         try:
             fn()
-        except Exception as msg:
+        except Exception as msg:    # pylint: disable=broad-except
+            self.errorsCount += 1
             if self.errorsAreFatal:
                 raise
             else:
-                print (msg)
+                print(msg, file=sys.stderr)
 
     def _isBlankOrComment(self, line):
         return line.isspace() or line == "" or line.lstrip()[0] == '#'
 
+    def _handleInclude(self, f):
+        # This case comes up primarily in ksvalidator.
+        if not self.followIncludes:
+            return
+
+        self._includeDepth += 1
+
+        try:
+            self.readKickstart(f, reset=False)
+        except KickstartError:
+            # Handle the include file being provided over the
+            # network in a %pre script.  This case comes up in the
+            # early parsing in anaconda.
+            if self.missingIncludeIsFatal:
+                raise
+
+        self._includeDepth -= 1
+
     def _stateMachine(self, lineIter):
         # For error reporting.
         lineno = 0
@@ -605,7 +741,7 @@ class KickstartParser:
         while True:
             # Get the next line out of the file, quitting if this is the last line.
             try:
-                self._line = lineIter.next()
+                self._line = next(lineIter)
                 if self._line == "":
                     break
             except StopIteration:
@@ -618,32 +754,14 @@ class KickstartParser:
                 self._handleSpecialComments(self._line)
                 continue
 
-            # Remove any end-of-line comments.
-            sanitized = self._line.split("#")[0]
-
-            # Then split the line.
-            args = shlex.split(sanitized.rstrip())
+            # Split the line, discarding comments.
+            args = shlex.split(self._line, comments=True)
 
             if args[0] == "%include":
-                # This case comes up primarily in ksvalidator.
-                if not self.followIncludes:
-                    continue
-
                 if len(args) == 1 or not args[1]:
-                    raise KickstartParseError (formatErrorMsg(lineno))
+                    raise KickstartParseError(lineno=lineno)
 
-                self._includeDepth += 1
-
-                try:
-                    self.readKickstart(args[1], reset=False)
-                except KickstartError:
-                    # Handle the include file being provided over the
-                    # network in a %pre script.  This case comes up in the
-                    # early parsing in anaconda.
-                    if self.missingIncludeIsFatal:
-                        raise
-
-                self._includeDepth -= 1
+                self._handleInclude(args[1])
                 continue
 
             # Now on to the main event.
@@ -656,7 +774,15 @@ class KickstartParser:
                     # here.
                     newSection = args[0]
                     if not self._validState(newSection):
-                        raise KickstartParseError (formatErrorMsg(lineno, msg=_("Unknown kickstart section: %s" % newSection)))
+                        if self.unknownSectionIsFatal:
+                            raise KickstartParseError(_("Unknown kickstart section: %s") % newSection, lineno=lineno)
+                        else:
+                            # If we are ignoring unknown section errors, just create a new
+                            # NullSection for the header we just saw.  Then nothing else
+                            # needs to change.  You can turn this warning into an error via
+                            # ksvalidator, or the warnings module.
+                            warnings.warn(_("Potentially unknown section seen at line %(lineno)s: %(sectionName)s") % {"lineno": lineno, "sectionName": newSection}, KickstartParseWarning)
+                            self.registerSection(NullSection(self.handler, sectionOpen=newSection))
 
                     self._state = newSection
                     obj = self._sections[self._state]
@@ -671,8 +797,12 @@ class KickstartParser:
                     self._tryFunc(lambda: self.handleCommand(lineno, args))
             elif self._state == STATE_END:
                 break
+            elif self._includeDepth > 0:
+                lineIter.put(self._line)
+                lineno -= 1
+                lineno = self._readSection(lineIter, lineno)
 
-    def readKickstartFromString (self, s, reset=True):
+    def readKickstartFromString(self, s, reset=True):
         """Process a kickstart file, provided as the string str."""
         if reset:
             self._reset()
@@ -681,7 +811,7 @@ class KickstartParser:
         # file reader and we only get StopIteration when we're after the final
         # line of input.
         i = PutBackIterator(s.splitlines(True) + [""])
-        self._stateMachine (i)
+        self._stateMachine(i)
 
     def readKickstart(self, f, reset=True):
         """Process a kickstart file, given by the filename f."""
@@ -694,7 +824,7 @@ class KickstartParser:
         # the reading "smart" by keeping track of what the path is at each
         # include depth.
         if not os.path.exists(f):
-            if self.currentdir.has_key(self._includeDepth - 1):
+            if self._includeDepth - 1 in self.currentdir:
                 if os.path.exists(os.path.join(self.currentdir[self._includeDepth - 1], f)):
                     f = os.path.join(self.currentdir[self._includeDepth - 1], f)
 
@@ -704,9 +834,9 @@ class KickstartParser:
         self.currentdir[self._includeDepth] = cd
 
         try:
-            s = urlread(f)
-        except grabber.URLGrabError as e:
-            raise KickstartError (formatErrorMsg(0, msg=_("Unable to open input kickstart file: %s") % e.strerror))
+            s = load_to_str(f)
+        except KickstartError as e:
+            raise KickstartError(_("Unable to open input kickstart file: %s") % str(e), lineno=0)
 
         self.readKickstartFromString(s, reset=False)
 
@@ -719,10 +849,16 @@ class KickstartParser:
 
         # Install the sections all kickstart files support.
         self.registerSection(PreScriptSection(self.handler, dataObj=Script))
+        self.registerSection(PreInstallScriptSection(self.handler, dataObj=Script))
         self.registerSection(PostScriptSection(self.handler, dataObj=Script))
+        self.registerSection(OnErrorScriptSection(self.handler, dataObj=Script))
         self.registerSection(TracebackScriptSection(self.handler, dataObj=Script))
         self.registerSection(RunScriptSection(self.handler, dataObj=Script))
         self.registerSection(PostUmountScriptSection(self.handler, dataObj=Script))
         self.registerSection(PackageSection(self.handler))
         self.registerSection(TpkPackageSection(self.handler))
 
+        # Whitelist well-known sections that pykickstart does not understand,
+        # but shouldn't error on.
+        self.registerSection(NullSection(self.handler, sectionOpen="%addon"))
+        self.registerSection(NullSection(self.handler, sectionOpen="%anaconda"))
