@@ -1,4 +1,4 @@
-#!/usr/bin/python -tt
+#!/usr/bin/python3 -tt
 # vim: ai ts=4 sts=4 et sw=4
 #
 # Copyright (c) 2012 Intel, Inc.
@@ -82,9 +82,10 @@ def main(parser, args, argv):
             rpm.setVerbosity(rpm.RPMLOG_DEBUG)
         except ImportError:
             pass
+
     #check the imager type
     createrClass = None
-    for subcmd, klass in pluginmgr.get_plugins('imager').iteritems():
+    for subcmd, klass in list(pluginmgr.get_plugins('imager').items()):
         if subcmd == args.subcommand and hasattr(klass, 'do_create'):
             createrClass = klass
 
@@ -135,7 +136,7 @@ def main(parser, args, argv):
     if args.strict_mode:
       configmgr.create['strict_mode'] = args.strict_mode
     if args.arch is not None:
-        supported_arch = sorted(rpmmisc.archPolicies.keys(), reverse=True)
+        supported_arch = sorted(list(rpmmisc.archPolicies.keys()), reverse=True)
         if args.arch in supported_arch:
             configmgr.create['arch'] = args.arch
         else:
@@ -149,8 +150,12 @@ def main(parser, args, argv):
 
     if args.skip_set_hosts:
         configmgr.create['skip_set_hosts']=args.skip_set_hosts
+
     if args.postscripts_maxruntime:
         configmgr.create['postscripts_maxruntime']=int(args.postscripts_maxruntime)
+    if args.block_recommends:
+        configmgr.create['block_recommends']=args.block_recommends
+
     if args.runtime:
         configmgr.set_runtime(args.runtime)
 
@@ -265,4 +270,3 @@ def do_auto(parser, ksfile, argv):
         args = parser.parse_args(options)
 
         main(parser, args, options)
-
