@@ -1,4 +1,4 @@
-#!/usr/bin/python -tt
+#!/usr/bin/python3 -tt
 #
 # Copyright (c) 2011 Intel, Inc.
 #
@@ -52,7 +52,7 @@ def save_mountpoints(fpath, loops, arch = None):
     for loop in loops:
         part = doc.createElement("partition")
         imgroot.appendChild(part)
-        for (key, val) in loop.items():
+        for (key, val) in list(loop.items()):
             if isinstance(val, fs.Mount):
                 continue
             part.setAttribute(key, str(val))
@@ -77,7 +77,7 @@ def load_mountpoints(fpath):
         dom = minidom.parse(rf)
     imgroot = dom.documentElement
     for part in imgroot.getElementsByTagName("partition"):
-        p  = dict(part.attributes.items())
+        p  = dict(list(part.attributes.items()))
 
         try:
             mp = (p['mountpoint'], p['label'], p['name'],
@@ -128,7 +128,7 @@ class LoopImageCreator(BaseImageCreator):
                                                        "ext3")
             self.__fsopts = kickstart.get_image_fsopts(self.ks,
                                                        "defaults,noatime")
-            if self.__fstype in AFTER_MNT_FS.keys():
+            if self.__fstype in list(AFTER_MNT_FS.keys()):
                 self.__fstype = "ext4"
 
             allloops = []
@@ -137,7 +137,7 @@ class LoopImageCreator(BaseImageCreator):
                 aft_fstype = None
                 if part.fstype == "swap":
                     continue
-                elif part.fstype in AFTER_MNT_FS.keys():
+                elif part.fstype in list(AFTER_MNT_FS.keys()):
                     aft_fstype = part.fstype
                     part.fstype = "ext4"
 
@@ -205,7 +205,7 @@ class LoopImageCreator(BaseImageCreator):
         names = []
         for lo in self._instloops :
             names.append(lo['name'])
-            for ro in AFTER_MNT_FS.values():
+            for ro in list(AFTER_MNT_FS.values()):
                 names.append(lo['name'].replace('.img',ro))
         return list(set(names))
 
@@ -438,7 +438,7 @@ class LoopImageCreator(BaseImageCreator):
         for item in self._instloops:
             imgfile = os.path.join(self._imgdir, item['name'])
 
-            if item['aft_fstype'] in AFTER_MNT_FS.keys():
+            if item['aft_fstype'] in list(AFTER_MNT_FS.keys()):
                 mountpoint = misc.mkdtemp()
                 ext4img = os.path.join(self._imgdir, item['name'])
                 runner.show('mount -t ext4 %s %s' % (ext4img, mountpoint))
