@@ -1,4 +1,4 @@
-#!/usr/bin/python -tt
+#!/usr/bin/python3 -tt
 #
 # Copyright (c) 2007 Red Hat, Inc.
 # Copyright (c) 2009, 2010, 2011 Intel, Inc.
@@ -33,7 +33,7 @@ from pykickstart.handlers.control import dataMap
 
 from mic import msger
 from mic.utils import errors, misc, runner, fs_related as fs
-from custom_commands import desktop, micrepo, micboot, partition, installerfw
+from .custom_commands import desktop, micrepo, micboot, partition, installerfw
 from mic.utils.safeurl import SafeURL
 
 AUTH_URL_PTN = r"(?P<scheme>.*)://(?P<username>.*)(:?P<password>.*)?@(?P<url>.*)"
@@ -321,7 +321,7 @@ class UserConfig(KickstartConfig):
     def addUser(self, userconfig):
         args = [ "/usr/sbin/useradd" ]
         if userconfig.groups:
-            args += [ "--groups", string.join(userconfig.groups, ",") ]
+            args += [ "--groups", ",".join(userconfig.groups) ]
         if userconfig.name:
             args += [ "-m"]
             args += [ "-d", "/home/%s" % userconfig.name  ]
@@ -357,7 +357,7 @@ class UserConfig(KickstartConfig):
             if userconfig.name == "root":
                 msger.debug("root user, just set root user groups attribution")
                 if userconfig.groups:
-                    self.call(["/usr/sbin/usermod", "-a", "-G", "%s" % string.join(userconfig.groups, ","), "root"])
+                    self.call(["/usr/sbin/usermod", "-a", "-G", "%s" % ",".join(userconfig.groups), "root"])
                 continue
             self.addUser(userconfig)
 
@@ -386,7 +386,7 @@ class XConfig(KickstartConfig):
         if ksxconfig.defaultdesktop:
             self._check_sysconfig()
             f = open(self.path("/etc/sysconfig/desktop"), "w")
-            f.write("DESKTOP="+ksxconfig.defaultdesktop+"\n")
+            f.write("DESKTOP=" + str(ksxconfig.defaultdesktop) + "\n")
             f.close()
 
 class DesktopConfig(KickstartConfig):
@@ -396,28 +396,28 @@ class DesktopConfig(KickstartConfig):
         if ksdesktop.defaultdesktop:
             self._check_sysconfig()
             f = open(self.path("/etc/sysconfig/desktop"), "w")
-            f.write("DESKTOP="+ksdesktop.defaultdesktop+"\n")
+            f.write("DESKTOP=" + str(ksdesktop.defaultdesktop) + "\n")
             f.close()
             if os.path.exists(self.path("/etc/gdm/custom.conf")):
                 f = open(self.path("/etc/skel/.dmrc"), "w")
                 f.write("[Desktop]\n")
-                f.write("Session="+ksdesktop.defaultdesktop.lower()+"\n")
+                f.write("Session=" + str(ksdesktop.defaultdesktop.lower()) + "\n")
                 f.close()
         if ksdesktop.session:
             if os.path.exists(self.path("/etc/sysconfig/uxlaunch")):
                 f = open(self.path("/etc/sysconfig/uxlaunch"), "a+")
-                f.write("session="+ksdesktop.session.lower()+"\n")
+                f.write("session=" + str(ksdesktop.session.lower()) + "\n")
                 f.close()
         if ksdesktop.autologinuser:
             self._check_sysconfig()
             f = open(self.path("/etc/sysconfig/desktop"), "a+")
-            f.write("AUTOLOGIN_USER=" + ksdesktop.autologinuser + "\n")
+            f.write("AUTOLOGIN_USER=" + str(ksdesktop.autologinuser) + "\n")
             f.close()
             if os.path.exists(self.path("/etc/gdm/custom.conf")):
                 f = open(self.path("/etc/gdm/custom.conf"), "w")
                 f.write("[daemon]\n")
                 f.write("AutomaticLoginEnable=true\n")
-                f.write("AutomaticLogin=" + ksdesktop.autologinuser + "\n")
+                f.write("AutomaticLogin=" + str(ksdesktop.autologinuser) + "\n")
                 f.close()
 
 class MoblinRepoConfig(KickstartConfig):
@@ -540,7 +540,7 @@ class NetworkConfig(KickstartConfig):
     def write_ifcfg(self, network):
         p = self.path("/etc/sysconfig/network-scripts/ifcfg-" + network.device)
 
-        f = file(p, "w+")
+        f = open(p, "w+")
         os.chmod(p, 0o644)
 
         f.write("DEVICE=%s\n" % network.device)
@@ -581,14 +581,14 @@ class NetworkConfig(KickstartConfig):
             return
 
         p = self.path("/etc/sysconfig/network-scripts/keys-" + network.device)
-        f = file(p, "w+")
+        f = open(p, "w+")
         os.chmod(p, 0o600)
         f.write("KEY=%s\n" % network.wepkey)
         f.close()
 
     def write_sysconfig(self, useipv6, hostname, gateway):
         path = self.path("/etc/sysconfig/network")
-        f = file(path, "w+")
+        f = open(path, "w+")
         os.chmod(path, 0o644)
 
         f.write("NETWORKING=yes\n")
@@ -618,7 +618,7 @@ class NetworkConfig(KickstartConfig):
         localline += "localhost.localdomain localhost"
 
         path = self.path("/etc/hosts")
-        f = file(path, "w+")
+        f = open(path, "w+")
         os.chmod(path, 0o644)
         f.write("127.0.0.1\t\t%s\n" % localline)
         f.write("::1\t\tlocalhost6.localdomain6 localhost6\n")
@@ -629,7 +629,7 @@ class NetworkConfig(KickstartConfig):
             return
 
         path = self.path("/etc/resolv.conf")
-        f = file(path, "w+")
+        f = open(path, "w+")
         os.chmod(path, 0o644)
 
         for ns in (nameservers):
@@ -795,13 +795,13 @@ def get_repos(ks, repo_urls=None, ignore_ksrepo=False):
     if repo_urls:
         if ignore_ksrepo:
             repos = {}
-        for name, repo in repo_urls.items():
+        for name, repo in list(repo_urls.items()):
             if 'baseurl' in repo:
                 repo['baseurl'] = SafeURL(repo.get('baseurl'),
                                           repo.get('user', None),
                                           repo.get('passwd', None))
             repos[name] = Repo(**repo)
-    return repos.values()
+    return list(repos.values())
 
 TpkRepoType = collections.namedtuple("TpkRepo",
                "name, baseurl,proxy,proxy_username,proxy_password,ssl_verify")
@@ -824,7 +824,7 @@ def get_tpkrepos(ks):
         if 'name' in tpkrepo:
             tpkrepos[tpkrepo['name']] = TpkRepo(**tpkrepo)
 
-    return tpkrepos.values()
+    return list(tpkrepos.values())
 
 
 def convert_method_to_repo(ks):
@@ -873,7 +873,7 @@ def inst_langs(ks):
 def get_post_scripts(ks):
     scripts = []
     for s in ks.handler.scripts:
-        if s.type != ksparser.KS_SCRIPT_POST:
+        if s.type != ksparser.constants.KS_SCRIPT_POST:
             continue
         scripts.append(s)
     return scripts
@@ -881,8 +881,8 @@ def get_post_scripts(ks):
 def get_sign_scripts(ks):
     scripts = []
     for s in ks.handler.scripts:
-        if (s.type == ksparser.KS_SCRIPT_RUN or \
-            s.type == ksparser.KS_SCRIPT_UMOUNT):
+        if (s.type == ksparser.constants.KS_SCRIPT_RUN or \
+            s.type == ksparser.constants.KS_SCRIPT_UMOUNT):
             scripts.append(s)
     return scripts
 
