@@ -1,4 +1,4 @@
-#!/usr/bin/python -tt
+#!/usr/bin/python3 -tt
 #
 # Copyright (c) 2009, 2010, 2011 Intel, Inc.
 #
@@ -116,8 +116,8 @@ def bootstrap_mic(argv=None):
             rootdir = os.path.join(rootdir, "bootstrap")
 
         bsenv.dirsetup(rootdir)
-        #when mic in mic-bootstrap is py3, use mic version in mic-bootstrap.
-        if cropts['use_mic_in_bootstrap'] | (mic_py2_in_mic_bootstrap(os.path.join(rootdir, '/usr/bin/mic'.lstrip('/'))) == False):
+        #when mic in mic-bootstrap is py2, use mic version in mic-bootstrap.
+        if cropts['use_mic_in_bootstrap'] | (mic_py2_in_mic_bootstrap(os.path.join(rootdir, '/usr/bin/mic'.lstrip('/'))) == True):
             msger.info("No copy host mic")
         else:
             msger.info("Copy host mic to bootstrap")
@@ -164,8 +164,8 @@ def get_bindmounts(cropts):
     for ltpkrepo in cropts['localtpkrepos']:
         binddirs.append(ltpkrepo)
 
-    bindlist = map(expath, filter(None, binddirs))
-    bindlist += map(os.path.dirname, map(expath, filter(None, bindfiles)))
+    bindlist = list(map(expath, [_f for _f in binddirs if _f]))
+    bindlist += list(map(os.path.dirname, list(map(expath, [_f for _f in bindfiles if _f]))))
     bindlist = sorted(set(bindlist))
     bindmounts = ';'.join(bindlist)
     return bindmounts
@@ -210,7 +210,7 @@ def get_mic_libpath():
 def sync_mic(bootstrap, binpth = '/usr/bin/mic',
              libpth='/usr/lib',
              plugin='/usr/lib/mic/plugins',
-             pylib = '/usr/lib/python2.7/site-packages',
+             pylib = '/usr/lib/python3/site-packages',
              conf = '/etc/mic/mic.conf'):
     _path = lambda p: os.path.join(bootstrap, p.lstrip('/'))
 
@@ -223,7 +223,7 @@ def sync_mic(bootstrap, binpth = '/usr/bin/mic',
                }
 
     if not os.path.exists(_path(pylib)):
-        pyptn = '/usr/lib/python?.?/site-packages'
+        pyptn = '/usr/lib/python3.*/site-packages'
         pylibs = glob.glob(_path(pyptn))
         if pylibs:
             pylib = pylibs[0].replace(bootstrap, '')
@@ -231,14 +231,14 @@ def sync_mic(bootstrap, binpth = '/usr/bin/mic',
             raise errors.BootstrapError("Can't find python site dir in: %s" %
                                         bootstrap)
 
-    for key, value in micpaths.items():
+    for key, value in list(micpaths.items()):
         try:
             safecopy(value, _path(eval(key)), False, ["*.pyc", "*.pyo"])
         except (OSError, IOError) as err:
             raise errors.BootstrapError(err)
 
     # auto select backend
-    conf_str = file(_path(conf)).read()
+    conf_str = open(_path(conf)).read()
     conf_str = re.sub("pkgmgr\s*=\s*.*", "pkgmgr=auto", conf_str)
     with open(_path(conf), 'w') as wf:
         wf.write(conf_str)
@@ -246,13 +246,6 @@ def sync_mic(bootstrap, binpth = '/usr/bin/mic',
     # chmod +x /usr/bin/mic
     os.chmod(_path(binpth), 0o777)
 
-    # correct python interpreter
-    mic_cont = file(_path(binpth)).read()
-    mic_cont = "#!/usr/bin/python\n" + mic_cont
-    with open(_path(binpth), 'w') as wf:
-        wf.write(mic_cont)
-
-
 def safecopy(src, dst, symlinks=False, ignore_ptns=()):
     if os.path.isdir(src):
         if os.path.isdir(dst):
@@ -316,7 +309,7 @@ def prepare_create(args):
                 pkgmgr = backends[key]
                 break
     else:
-        for key in backends.keys():
+        for key in list(backends.keys()):
             if key == creatoropts['pkgmgr']:
                 pkgmgr = backends[key]
                 break
@@ -325,6 +318,6 @@ def prepare_create(args):
         raise errors.CreatorError("Can't find backend: %s, "
                                   "available choices: %s" %
                                   (creatoropts['pkgmgr'],
-                                   ','.join(backends.keys())))
+                                   ','.join(list(backends.keys()))))
     return creatoropts, pkgmgr, recording_pkgs
 
