@@ -1,4 +1,4 @@
-#!/usr/bin/python -tt
+#!/usr/bin/python3 -tt
 #
 # Copyright (c) 2007 Red Hat  Inc.
 # Copyright (c) 2010, 2011 Intel, Inc.
@@ -121,7 +121,7 @@ class Yum(BackendPlugin, yum.YumBase):
         self.strict_mode = strict_mode
 
         if self.target_arch:
-            if not rpmUtils.arch.arches.has_key(self.target_arch):
+            if self.target_arch not in rpmUtils.arch.arches:
                 rpmUtils.arch.arches["armv7hl"] = "noarch"
                 rpmUtils.arch.arches["armv7tnhl"] = "armv7nhl"
                 rpmUtils.arch.arches["armv7tnhl"] = "armv7thl"
@@ -161,7 +161,7 @@ class Yum(BackendPlugin, yum.YumBase):
     def _writeConf(self, confpath, installroot):
         conf = Template(YUMCONF_TEMP).safe_substitute(installroot=installroot)
 
-        f = file(confpath, "w+")
+        f = open(confpath, "w+")
         f.write(conf)
         f.close()
 
@@ -233,22 +233,22 @@ class Yum(BackendPlugin, yum.YumBase):
                 # we also need to remove from the conditionals
                 # dict so that things don't get pulled back in as a result
                 # of them.  yes, this is ugly.  conditionals should die.
-                for req, pkgs in self.tsInfo.conditionals.iteritems():
+                for req, pkgs in list(self.tsInfo.conditionals.items()):
                     if x in pkgs:
                         pkgs.remove(x)
                         self.tsInfo.conditionals[req] = pkgs
         else:
             msger.warning("No such package %s to remove" %(pkg,))
 
-    def selectGroup(self, grp, include = ksparser.GROUP_DEFAULT):
+    def selectGroup(self, grp, include = ksparser.constants.GROUP_DEFAULT):
         try:
             yum.YumBase.selectGroup(self, grp)
-            if include == ksparser.GROUP_REQUIRED:
-                for p in grp.default_packages.keys():
+            if include == ksparser.constants.GROUP_REQUIRED:
+                for p in list(grp.default_packages.keys()):
                     self.deselectPackage(p)
 
-            elif include == ksparser.GROUP_ALL:
-                for p in grp.optional_packages.keys():
+            elif include == ksparser.constants.GROUP_ALL:
+                for p in list(grp.optional_packages.keys()):
                     self.selectPackage(p)
 
             return None
@@ -283,7 +283,7 @@ class Yum(BackendPlugin, yum.YumBase):
             repo.mirrorlist = _varSubstitute(mirrorlist)
 
         conf = yum.config.RepoConf()
-        for k, v in conf.iteritems():
+        for k, v in list(conf.items()):
             if v or not hasattr(repo, k):
                 repo.setAttribute(k, v)
 
@@ -317,11 +317,7 @@ class Yum(BackendPlugin, yum.YumBase):
 
     def installHasFile(self, file):
         provides_pkg = self.whatProvides(file, None, None)
-        dlpkgs = map(
-                    lambda x: x.po,
-                    filter(
-                        lambda txmbr: txmbr.ts_state in ("i", "u"),
-                        self.tsInfo.getMembers()))
+        dlpkgs = [x.po for x in [txmbr for txmbr in self.tsInfo.getMembers() if txmbr.ts_state in ("i", "u")]]
 
         for p in dlpkgs:
             for q in provides_pkg:
@@ -342,11 +338,7 @@ class Yum(BackendPlugin, yum.YumBase):
             raise CreatorError("Failed to build transaction : %s" \
                                % str.join("\n", resmsg))
 
-        dlpkgs = map(
-                    lambda x: x.po,
-                    filter(
-                        lambda txmbr: txmbr.ts_state in ("i", "u"),
-                        self.tsInfo.getMembers()))
+        dlpkgs = [x.po for x in [txmbr for txmbr in self.tsInfo.getMembers() if txmbr.ts_state in ("i", "u")]]
 
         # record all pkg and the content
         for pkg in dlpkgs:
@@ -358,7 +350,7 @@ class Yum(BackendPlugin, yum.YumBase):
                             }
             self.__pkgs_content[pkg_long_name] = pkg.files
             license = pkg.license
-            if license in self.__pkgs_license.keys():
+            if license in list(self.__pkgs_license.keys()):
                 self.__pkgs_license[license].append(pkg_long_name)
             else:
                 self.__pkgs_license[license] = [pkg_long_name]
@@ -371,12 +363,12 @@ class Yum(BackendPlugin, yum.YumBase):
 
         total_count = len(dlpkgs)
         cached_count = 0
-        download_total_size = sum(map(lambda x: int(x.packagesize), dlpkgs))
+        download_total_size = sum([int(x.packagesize) for x in dlpkgs])
 
         msger.info("\nChecking packages cached ...")
         for po in dlpkgs:
             local = po.localPkg()
-            repo = filter(lambda r: r.id == po.repoid, self.repos.listEnabled())[0]
+            repo = list(filter(lambda r: r.id == po.repoid, self.repos.listEnabled()))[0]
             if repo.nocache and os.path.exists(local):
                 os.unlink(local)
             if not os.path.exists(local):
@@ -483,7 +475,7 @@ class Yum(BackendPlugin, yum.YumBase):
         if not pkgname:
             return None
 
-        pkg = filter(lambda txmbr: txmbr.po.name == pkgname, self.tsInfo.getMembers())
+        pkg = [txmbr for txmbr in self.tsInfo.getMembers() if txmbr.po.name == pkgname]
         if not pkg:
             return None
         return pkg[0].po.filelist
